Enum demes_forward::DemesForwardError
source · pub enum DemesForwardError {
DemesError(DemesError),
TimeError(String),
InvalidDemeSize(DemeSize),
InternalError(String),
}Expand description
Error type.
Variants
DemesError(DemesError)
Stores a demes::DemesError.
TimeError(String)
Errors related to time. Will be returned if invalid time values occur after converting time to generations.
InvalidDemeSize(DemeSize)
Errors related to invalid deme sizes arising during application of size change functions.
InternalError(String)
Errors related to invalid internal states. In general, this error indicates a bug that should be reported.