pub struct ForwardGraph { /* private fields */ }
Forward-time representation of a demes::Graph.

Implementations

Constructor

Parameters

Update the internal state of the graph to the parental generation time parental_generation_time.

The total number of demes in the graph.

The ancestry proporitions for a given offspring deme at the current time.

Parameters
  • offspring_deme: the index of an offspring deme.
Returns
  • Some(&[f64]) if offspring_deme is a valid index and extant offspring demes exist.
  • None otherwise.

Get cloning rates of all offspring demes.

Returns None if there are no extant offspring demes.

Get selfing rates of all offspring demes.

Returns None if there are no extant offspring demes.

Obtain the time corresponding to the last call of ForwardGraph::update_state.

Obtain the end time of the model.

Return an iterator over time values.

The iterator starts at the last updated time and continues until the end time.

Obtain the sizes of each parental deme.

The length of the slice is equal to the number of demes in the graph (see ForwardGraph::num_demes_in_model).

Returns None if there are no parental demes at the current time.

Obtain the sizes of each offspring deme.

The length of the slice is equal to the number of demes in the graph (see ForwardGraph::num_demes_in_model).

Returns None if there are no offspring demes at the current time.

Return true if there are any extant parental demes at the current time.

Return true if there are any extant offspring demes at the current time.

Return the number of extant parental demes at the current time.

Return the number of extant offspring demes at the current time.